/*
 * File:   Lcd.c
 * Mp3_02c
 *
 * Created on 2015/05/02, 8:10
 */

#define LBL_BLANK       "                "
#define LBL_INIT_0      "MP3_02C         "
#define LBL_INIT_1      "2015/05/04      "
#define LBL_NO_SD       "NO SD CARD      "
#define LBL_SD_FOUND    "SD CARD FOUND   "
#define LBL_NO_FAT16    "NO FAT16        "
#define LBL_NO_MP3      "NO MP3          "
#define LBL_MP3_EXIST   "MP3 EXIST!      "
#define LCD_CURSOR_AT_HOME      0x02        // return the cursor to home position

void lcd_set_data_port(unsigned int data);
void lcd_write_data_clock();
void lcd_write_command(unsigned int data);
void lcd_wait();
void lcd_write_data(char data);
void lcd_locate(unsigned char line, unsigned char column);
void lcd_write_command_clock();

void lcd_set_data_port(unsigned int data){
    output_bit(LCD_D4, (data & 0x01));   //LCD_D4 = (data & 0x01);
    output_bit(LCD_D5, ((data & 0x02) >> 1));   //LCD_D5 = ((data & 0x02) >> 1);
    output_bit(LCD_D6, ((data & 0x04) >> 2));   //LCD_D6 = ((data & 0x04) >> 2);
    output_bit(LCD_D7, ((data & 0x08) >> 3));   //LCD_D7 = ((data & 0x08) >> 3);
}

void lcd_write_data_clock(){
    output_high(LCD_RS);      //LCD_RS = 1;
//    LCD_RW = 0;
    lcd_wait();     // >=60ns
    output_high(LCD_E);      //LCD_E = 1;
    lcd_wait();     // >=450ns
    output_low(LCD_E);       //LCD_E = 0;
    lcd_wait();     // >=10ns
}

void lcd_write_command(unsigned int data){
    lcd_set_data_port(data >> 4);
    lcd_write_command_clock();
    lcd_set_data_port(data);
    lcd_write_command_clock();
    wait_ms(10);
}

void lcd_write_command_clock(){
    output_low(LCD_RS);   //LCD_RS = 0;
//    LCD_RW = 0;
    lcd_wait();     // >=60ns
    output_high(LCD_E);  //LCD_E = 1;
    lcd_wait();     // >=450ns
    output_low(LCD_E);   //LCD_E = 0;
    lcd_wait();     // >=10ns
}

// Display starting message
void lcd_test(){
    lcd_write_command(LCD_CURSOR_AT_HOME);
    lcd_locate(0, 0); lcd_println(LBL_INIT_0);
    lcd_locate(1, 0); lcd_println(LBL_INIT_1);
//    lcd_locate(1, 0); lcd_display_hex("TST ", 0x1A2B3C4D);
}

void lcd_sd_ok(){
    lcd_write_command(LCD_CURSOR_AT_HOME);
    lcd_locate(0, 0); lcd_println("SD OK           ");
    lcd_locate(1, 0); lcd_println(LBL_BLANK);
}

void lcd_sd_ng(){
    lcd_write_command(LCD_CURSOR_AT_HOME);
    lcd_locate(0, 0); lcd_println("SD NG !?        ");
    lcd_locate(1, 0); lcd_println(LBL_BLANK);
}

void lcd_println(const unsigned char *str){
    //lcd_write_data(str);
    while (*str != '\0'){
        lcd_write_data(*str++);
    }
}

/*
 *  DDRAM address set
 */
void lcd_locate(unsigned char line, unsigned char column){
    /*
    unsigned int cmd;

    switch (line){
    case 0: cmd = 0x80; break;
    case 1: cmd = 0xC0; break;
    default: return;
    }
    lcd_write_command(cmd + column);
    */
    
    lcd_write_command((line==0 ? 0x80 : 0xc0) + column);
}

void lcd_write_data(char data){
    lcd_set_data_port(data >> 4);
    lcd_write_data_clock();
    lcd_set_data_port(data);
    lcd_write_data_clock();
    wait_ms(10);
}

void lcd_wait(){
    wait_ms(1);
}
