/*
 * File:   Lcd_I2C.c
 * Mp3_02c
 *
 * Created on 2015/05/22, 8:10
 */

#define LCD_ADDRESS         0x7C            // 0b0111 110 0
#define LCD_CONTRAST        0b110000
#define LCD_BLANK_LINE      "                "


//void lcd_test(){
//}


void i2c_start_lcd(){
    i2c_start();
    i2c_write(LCD_ADDRESS);
}

void lcd_init(){
    int8    ack;
    
    output_high(PIN_LCD_I2C_RST);
    delay_ms(1);
    
    i2c_start();
    ack = i2c_write(LCD_ADDRESS);
    //write_eeprom(0x20, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b00111000);        // function set(DL=1:8bit interface date, N=1:2lines)
    //write_eeprom(0x21, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b00111001);        // function set(DL=1:8bit interface date, N=1:2lines, IS=1:extension instruction)
    //write_eeprom(0x22, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b00010100);        // interval osc(BS=0:Bias 1/5, F210=100:Internal osc freq. adjust)
    //write_eeprom(0x23, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b01110000 | (LCD_CONTRAST & 0xF));        // contrast low
    //write_eeprom(0x24, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b01011100 | ((LCD_CONTRAST >> 4) & 0x3));  // contrast high,
    //write_eeprom(0x25, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b01101100);        // follower control
    //write_eeprom(0x26, ack);

    delay_ms(200);

    ack = i2c_write(0x80);
    ack = i2c_write(0b00111000);        // function set(DL=1:8bit interface date, N=1:2lines)
    //write_eeprom(0x27, ack);
    ack = i2c_write(0x80);
    ack = i2c_write(0b00001100);        // Display On (D=1:display on, C=0:cursor disapper, B=0:Blink off)
    //write_eeprom(0x28, ack);

    ack = i2c_write(0x00);
    //write_eeprom(0x29, ack);
    ack = i2c_write(0b00000001);        // Clear Display
    //write_eeprom(0x2A, ack);
    delay_ms(2);			// Clear Display

    i2c_stop();
}


/*
 *  DDRAM address set
 */
void lcd_locate(unsigned char line, unsigned char column){
    int8    ack;
    
    ack = i2c_write(0x80);      // Co=1, RS=0:Instruction Write
    ack = i2c_write((line==0 ? 0x80 : 0xc0) + column);      // Set DDRAM address
}

void lcd_nprintln(const unsigned char *str, unsigned char n)
{
    int8    ack;
    unsigned char *startp;

    startp = str;

    while(*str && (uint8_t)(str - startp + 1) <= n){
        ack = i2c_write(0xC0);  // Co=1, RS=1:Data Write
        ack = i2c_write(*str++);
    }
}

void lcd_println(const unsigned char *str)
{
    int8    ack;
    int     i = 0;

    while(*str && i < 16){
        ack = i2c_write(0xC0);
        ack = i2c_write(*str++);
        i++;
    }
}

void lcd_sd_ng()
{
    i2c_start_lcd();
    lcd_locate(0, 0); lcd_println("SD CARD ERROR   ");
    lcd_locate(1, 0); lcd_println("                ");
    i2c_stop();
}

void lcd_icon_for_subdirectory(uint16_t numsndfile, uint16_t recnosubdir)
{
    int8    ack;

    i2c_start_lcd();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x44);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (numsndfile != 0){
        ack = i2c_write(0x10);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x00
    }

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x47);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (recnosubdir > 1){
        ack = i2c_write(0x08);      // Data    : 0x10
    } else {
        ack = i2c_write(0x00);      // Data    : 0x10
    }
    
    i2c_stop();
}

/*
 *  Bass/Treble Control state icon
 */
void lcd_icon_bt_control(uint8_t bt_control)
{
    int8        ack;
    uint8_t     state;

    i2c_start_lcd();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x4D);      // Command : Set ICON address

    ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
    if (bt_control >= 1) state = 0x02;  // display "battery frame"
    else state = 0x00;
    if (bt_control >= 1) state |= 0x10;
    if (bt_control >= 2) state |= 0x08;
    if (bt_control >= 3) state |= 0x04;
    ack = i2c_write(state);      // Data    :

    i2c_stop();
}

#if 0
void lcd_set_icon()
{
    int8    ack;
    unsigned char i;

    i2c_start_lcd();
    ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
    ack = i2c_write(0x39);      // Command : Function Set(DL=1, N=1, DH=0, IS=1)

    for (i=0; i<16; i++){
        ack = i2c_write(0x80);      // Control : Co=1, RS=0:Instruction Write
        ack = i2c_write(0x40 + i);      // Command : Set ICON address

        ack = i2c_write(0xC0);      // Control : Co=1, RS=1:Data Write
        if (i == 0){
            ack = i2c_write(0x10);      // Data    : 0x10
        } else {
            ack = i2c_write(0x00);      // Data    : 0x00
        }
    }
    
    i2c_stop();
}
#endif

