/* 
 * File:   main04.c
 *
 * Created on 2015/06/07
 */

#include <18F2580.h>

#device ansi
#device const=read_only
#device PASS_STRINGS=IN_RAM

#opt compress

// 18F2580 (INTIO2 / port function on RA6 and RA7)
#fuses 1=0x0800     // IESO=0, FCMEN=0, INTIO2, BOREN=0, PWRT enabled 
#fuses NOWDT,NOPBADEN,NOMCLR,NOLVP,NOCPB,NOCPD
#use delay (clock=32000000)
#use I2C(MASTER, FORCE_SW, SCL=PIN_A4, SDA=PIN_A3, SLOW)

#include <stdio.h>
#include <stdlib.h>

#include "integer.h"
#include "diskio.h"
#include "ffconf.h"
#include "ff.h"
#include "user.h"

// timer1
#define TMR1REG_START       (3817)      // 0.05s(?)
#define CHATTERING_ADJUST   9          // 15
#define ERROR_WAIT_TIME     5000

#define PIN_POWER_ON        PIN_C1      // output
#define PIN_POWER_OFF       PIN_C2      // input
#define PIN_LED             PIN_A7      // output

#define MP3_SI              PIN_B1
#define MP3_SCLK            PIN_C7
#define MP3_CS              PIN_C6
#define MP3_BSYNC           PIN_B2
#define MP3_DCLK            PIN_B3
#define MP3_DREQ            PIN_B4
#define MP3_RS              PIN_B5

#define PIN_LCD_I2C_RST     PIN_A5

#define PIN_SWPLAY          PIN_A0
#define PIN_SWRIGHT         PIN_B6
#define PIN_SWLEFT          PIN_A1
#define PIN_SWUP            PIN_B0
#define PIN_SWDOWN          PIN_A2
#define PIN_SWTEMPO         PIN_E3
#define PIN_SWTREBLE        PIN_B7

#define sw_play()           input(PIN_SWPLAY)
#define sw_right()          input(PIN_SWRIGHT)
#define sw_left()           input(PIN_SWLEFT)
#define sw_up()             input(PIN_SWUP)
#define sw_down()           input(PIN_SWDOWN)
#define sw_tempo()          input(PIN_SWTEMPO)
#define sw_treble()         input(PIN_SWTREBLE)
#define sw_poff()           input(PIN_POWER_OFF)

#define SWPLAY_PRESSED      0x81
#define SWRIGHT_PRESSED     0x82
#define SWLEFT_PRESSED      0x83
#define SWUP_PRESSED        0x84
#define SWDOWN_PRESSED      0x85
#define SWTEMPO_PRESSED     0x86
#define SWTREBLE_PRESSED    0x87
#define SWPOFF_PRESSED      0x88

#define PLAY_NEXT_MP3       0x91
#define PLAY_PREV_MP3       0x92
#define PLAY_OTHER_DIR      0x93
#define POWEROFF            0x94

#define SEND_SIZE           32
#define MP3_BUFFER_SIZE     512           // 512=32*16
#define FF_REW_SIZE         (_MIN_SS * 200L)        // 512*200=102400=0.1MByte
#define EEPROM_ADDR_SEED    0
#define EEPROM_ADDR_VOL     4

#define MIN_VOLUME          0x00
#define MAX_VOLUME          0xD0
#define DEFAULT_VOLUME      0x30
#define STEP_VOLUME         0x04

//#define DIRLISTFILE         "/DIRLIST.TXT"      // dir /AD /ON /S /B > DIRLIST.TXT
// Input files (created on windows by DIRMP304.exe)
#define PITCHSHIFTERFILE        "/PSHFT131.DAT"     // PitchShifter data

// DIRLIST04FILE parameters
#define DIRLIST04FILE            "/DIRLST04.DAT"
#define RECLEN_DIRLIST04         128UL
#define MAXLEN_FILEPATH          60UL
#define MAXLEN_DIRPATH           47UL
#define DL_FULLDIR               0
#define DL_LENFULLDIR            61
#define DL_DIRNAME               62
#define DL_RECNO_SUBDIR_START    75
#define DL_NUM_SUBDIR            77
#define DL_RECNO_PARENTDIR       78
#define DL_RECNO_GRANDPARENTDIR  80
#define DL_RECNO_NEXTRANDOMDIR   82
#define DL_NUM_SNDFILE           84
#define DL_DISPLAYNAME           85

// SNDFILELIST04FILE parameters
#define SNDFILELIST04FILE        "SNDLST04.DAT"
#define RECLEN_MP3LIST04         64UL
#define ML_FILENAME              0
#define ML_TRACKNO               13
#define ML_TITLENAME             16

// VOLUME04FILE parameters
#define VOLUME04FILE             "VOLUME04.DAT"
#define RECLEN_VOLUME04          2UL
#define VF_VOLUME                0
#define VF_COUNT                 1

// HISTFILE parameters
#define HIST04FILE                "/HIST04.DAT"
#define RECLEN_HISTFILE04         10UL
//#define MAX_HIST_SIZE             100
#define HS_LASTDIR                0
#define HS_LASTTRACK              2
#define HS_LASTBYTE               4
#define HS_NEXTRANDOMDIR          8

// error log
#define ERRLOG04FILE              "/ERRLOG04.DAT"
#define RECLEN_ERRLOG04           8UL
#define MAX_ERRLOG_COUNT          1000
#define EL_ID                   0
#define EL_ERRORCODE            1
#define EL_ERRORVALUE           2
#define EL_DIRNO                3
#define EL_TRACKNO              5
#define ERRORCODE_OPEN_MP3LISTFILE        1
#define ERRORCODE_OPEN_MP3FILE            2
#define ERRORCODE_SAVE_HISTORYFILE        3

#define wait_ms(m)  delay_ms(m)

#define SUPPORT_RANDOM_TRACK

static FATFS        Fatfs;
static BOOLEAN      EVENT_SWANY;
static BOOLEAN      EVENT_SWPLAY;
static BOOLEAN      EVENT_SWRIGHT;
static BOOLEAN      EVENT_SWLEFT;
static BOOLEAN      EVENT_SWUP;
static BOOLEAN      EVENT_SWDOWN;
static BOOLEAN      EVENT_SWTEMPO;
static BOOLEAN      EVENT_SWTREBLE;
static BOOLEAN      EVENT_SWPOFF;
static uint16_t     CHATTERING_TMR;
static uint16_t     SW_CONTINUE_COUNT;
static BOOLEAN      SwplayLast;

static uint8_t      VOLUME;
static uint16_t     ST_AMP, ST_FREQ, SB_AMP, SB_FREQ;   // bass/treble control values
static uint8_t      BTControl;      // Bass/Treble control
static uint8_t      PitchShifterMode;
static BOOLEAN      PonRestart;     // execute Power on Restart
static BOOLEAN      AutoPoff;
static uint16_t     AutoPoffTmr;

enum playmode_t {PM_RANDOM_DIR, PM_RANDOM_TRACK, PM_SELECT};
static playmode_t      PlayMode;

// PlayOrSelectMp3InSubdirectory function
enum playorselect_t {PS_PLAY, PS_SELECT};

static uint8_t      PlayDirHist[RECLEN_HISTFILE04];
static uint8_t      LogId;

int8 PauseAndSelectItem();
int8 SelectDirectory(char *RecDirList, FIL *FilDL);
int8 PlayRandom(char *RecDirList, FIL *FilDL);
int8 PlayOrSelectMp3InSubdirectory(char *RecDirList, uint16_t dirno, uint16_t numsndfile, uint8_t start_trackno, playorselect_t playorselect);
int8 PlayOneMp3(const char *mp3file, uint16_t dirno, uint8_t trackno, const char *DirListRec, const char *Mp3ListRec);
void DisplayDirectoryName(char *RecDirList);
void DisplayTitleName(char *RecMp3List);
void InitApp(void);
uint16_t ReadEepromUint16(uint8_t address);
void WriteEepromUint16(uint8_t address, uint16_t data);
uint32_t ReadEepromUint32(uint8_t address);
void WriteEepromUint32(uint8_t address, uint32_t data);
void InitPitchShifter(void);
void ReadHistory(FIL *Fil);
void SaveHistory(FIL *Fil);
void ErrorLog(uint8_t id, uint8_t errorcode, uint8_t errorvalue, uint16_t dirno, uint8_t trackno);
void DispNoDirList();
void DispNoMp3List();
void DispNoMp3();
void wait_ms(uint16_t ms);
void DispNumForDebug(uint32_t v);

#include "spi_sd.c"
#include "ff.c"
#include "Lcd_I2C.c"
#include "vs.c"

#INT_TIMER1
void isr_timer(){
    uint16_t n;

    n = Timer1;
    if (n) Timer1 = --n;
    n = Timer2;
    if (n) Timer2 = --n;

    if (AutoPoff == 1){
        AutoPoffTmr++;
    }

    if (CHATTERING_TMR > 0){
        CHATTERING_TMR++;
        if (CHATTERING_TMR == CHATTERING_ADJUST) CHATTERING_TMR = 0;
    } else {
        BOOLEAN    SwCurrent;

        // sw_play(PLAY button) supports long time press
        SwCurrent = sw_play();
        if (SwplayLast == SwCurrent){
            if (SwCurrent == 0){   // press continues
                SW_CONTINUE_COUNT++;
            } else {    // release continues
                SW_CONTINUE_COUNT = 0;
            }
        } else {    // press state change (press or release)
            CHATTERING_TMR = 1;     // chattering timer start
            if (SwCurrent == 1){    // released
                EVENT_SWPLAY = 1;   // press event happens
                EVENT_SWANY = 1;
            }
        }
        SwplayLast = SwCurrent;

#if 0
        if (sw_play() == 0 && EVENT_SWPLAY == 0){
            EVENT_SWPLAY = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
#endif
        if (sw_right() == 0 && EVENT_SWRIGHT == 0){
            EVENT_SWRIGHT = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_left() == 0 && EVENT_SWLEFT == 0){
            EVENT_SWLEFT = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_up() == 0 && EVENT_SWUP == 0){
            EVENT_SWUP = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_down() == 0 && EVENT_SWDOWN == 0){
            EVENT_SWDOWN = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_tempo() == 0 && EVENT_SWTEMPO == 0){
            EVENT_SWTEMPO = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_treble() == 0 && EVENT_SWTREBLE == 0){
            EVENT_SWTREBLE = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
        if (sw_poff() == 0 && EVENT_SWPOFF == 0){
            EVENT_SWPOFF = 1;
            EVENT_SWANY = 1;
            CHATTERING_TMR = 1;     // chattering timer start
        }
    }
}


/*
 * 
 */
int main(int argc, char** argv) {
    FRESULT     res;
    char        drive[8];
    int8        ret;
    FIL         FilDL;          // Directory list file
    FIL         FilHs;          // History file
    uint8_t     RecDirList[RECLEN_DIRLIST04];

    InitApp();
    EVENT_SWANY = 0;
    EVENT_SWPLAY = 0;
    EVENT_SWRIGHT = 0;
    EVENT_SWLEFT = 0;
    EVENT_SWUP = 0;
    EVENT_SWDOWN = 0;
    EVENT_SWTEMPO = 0;
    EVENT_SWTREBLE = 0;
    EVENT_SWPOFF = 0;
    CHATTERING_TMR = 0;
    SwplayLast = sw_play();
    PonRestart = 0;
    AutoPoff = 0;
    AutoPoffTmr = 0;
    
    VOLUME = read_eeprom(EEPROM_ADDR_VOL);
    if (!(VOLUME <= MAX_VOLUME && VOLUME >= MIN_VOLUME)){
        VOLUME = DEFAULT_VOLUME;
    }
    ST_AMP = 0x00; ST_FREQ = 0x00; SB_AMP = 0x00; SB_FREQ = 0x00;

    output_low(MP3_RS);
    wait_ms(100);
    output_high(MP3_RS);
    wait_ms(100);

    strcpy(drive, _T("0:"));
    res = f_mount(&Fatfs, drive, 0);
    if (res == FR_OK){
        lcd_icon_bt_control(BTControl);
        ReadHistory(&FilHs);

        do {
            i2c_start_lcd();
            lcd_locate(0, 0); lcd_println("PLAY -> RANDOM  ");
            lcd_locate(1, 0); lcd_println("DOWN -> SELECT  ");
            i2c_stop();

            ret = PauseAndSelectItem();
            if (ret == SWPLAY_PRESSED){
                PlayMode = PM_RANDOM_DIR;
                PonRestart = 1;
                ret = PlayRandom(RecDirList, &FilDL);
                if (ret != FR_OK) break;
#ifdef SUPPORT_RANDOM_TRACK
            } else if (ret == SWLEFT_PRESSED){
                PlayMode = PM_RANDOM_TRACK;
                ret = PlayRandom(RecDirList, &FilDL);
                if (ret != FR_OK) break;
#endif
            } else if (ret == SWDOWN_PRESSED){
                PlayMode = PM_SELECT;
                ret = SelectDirectory(RecDirList, &FilDL);
                if (ret != FR_OK) break;
            } else if (ret == SWPOFF_PRESSED){
                break;
            }
        } while (1);

        SaveHistory(&FilHs);
    }
    output_low(PIN_POWER_ON);
    return (0);
}

FRESULT f_read_DIRLIST04FILE (
	FIL* fp, 		/* Pointer to the file object */
	void* buff		/* Pointer to data buffer */
){
    FRESULT res1;
    UINT    br;
    res1 = f_read(fp, buff, RECLEN_DIRLIST04, &br);
    if (br != RECLEN_DIRLIST04) return FR_NO_FILE;
    return res1;
}

int8
PlayRandom(char *RecDirList, FIL *FilDL){
    FRESULT     res;
    int8        ret;
    uint16_t    numdirs;
    uint16_t    nummp3dirs;
    uint16_t    numsnds;
    uint16_t    dirno;
    uint8_t     start_trackno;
    uint32_t    ofs;
    uint16_t    lastdirno;

    ret = FR_OK;

    // random play start
    res = f_open(FilDL, DIRLIST04FILE, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        DispNoDirList();
        return res;
    }
    // header record
    res = f_read_DIRLIST04FILE(FilDL, RecDirList);
    if (res != FR_OK) return res;
    numdirs = *(uint16_t *)(&RecDirList[0]);
    nummp3dirs = *(uint16_t *)(&RecDirList[2]);

    while (1){
        if (PonRestart == 1){
            lastdirno = *(uint16_t *)(PlayDirHist + HS_LASTDIR);
            if (lastdirno >= 0 && lastdirno < numdirs){
                dirno = lastdirno;
                start_trackno = *(uint16_t *)(PlayDirHist + HS_LASTTRACK);
            } else {
                PonRestart = 0;
                continue;
            }
        } else {
            dirno = *(uint16_t *)(PlayDirHist + HS_NEXTRANDOMDIR);
#if 0
            dirno = rand() % numdirs;
            if (CheckHistory(dirno, nummp3dirs/2) == 1){
                continue;
            }
#endif
            start_trackno = 1;
        }
        ofs = (uint32_t)(dirno + 1) * (uint32_t)RECLEN_DIRLIST04;     // +1 = header record
        res = f_lseek(FilDL, ofs);
        if (res == FR_OK){
            res = f_read_DIRLIST04FILE(FilDL, RecDirList);
            if (res == FR_OK){
                numsnds = *(RecDirList + DL_NUM_SNDFILE);    // number of sound files
                if (numsnds > 0){
#if 0
                    if (PonRestart == 1){
                        lasttrackno = *(uint16_t *)(PlayDirHist + HS_LASTTRACK);
                        if (lasttrackno>=1 && lasttrackno <= numsnds){
                            start_trackno = lasttrackno;
                        } else {
                            start_trackno = 1;
                        }
                    } else {
                        start_trackno = 1;
                        AddDirnoToHistory(dirno);
                    }
#endif
                    if (start_trackno < 1 || start_trackno > numsnds){
                        start_trackno = 1;
                    }
                    if (PonRestart == 0){
                        *(uint16_t *)(PlayDirHist + HS_NEXTRANDOMDIR) = *(uint16_t *)(RecDirList + DL_RECNO_NEXTRANDOMDIR);
//                        AddDirnoToHistory(dirno);
                    }
                    ret = PlayOrSelectMp3InSubdirectory(RecDirList, dirno, numsnds, start_trackno, PS_PLAY);
                    if (ret == POWEROFF){
                        break;
                    }
                }
            }
        }
        PonRestart = 0;
    }
    f_close(FilDL);

    return (ret);
}

int8
SelectDirectory(char *RecDirList, FIL *FilDL){
    FRESULT     res;
    uint32_t    ofsSubDirStart;
    uint16_t    rowSubDirStart;
    uint16_t    numSubDir;
    uint16_t    idxSubDir;
    int8        ret;
    uint8_t     idxDirDepth;
    uint16_t    stackIdxSubDir[8];
    uint16_t    dirno;
    uint16_t    numSndFile;
    uint16_t    recNoFirstSubdir;

    ret = FR_OK;

    res = f_open(FilDL, DIRLIST04FILE, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        DispNoDirList();
        return res;
    }
    // Read HEADER info
    res = f_read_DIRLIST04FILE(FilDL, RecDirList);
    if (res != FR_OK){
        return res;
    }
    // Read ROOT info
    res = f_read_DIRLIST04FILE(FilDL, RecDirList);
    if (res != FR_OK){
        return res;
    }
    rowSubDirStart = *((uint16_t *)(RecDirList + DL_RECNO_SUBDIR_START));
    ofsSubDirStart = (uint32_t)rowSubDirStart * RECLEN_DIRLIST04;
    numSubDir = *(RecDirList + DL_NUM_SUBDIR);
    idxSubDir = 0;
    idxDirDepth = 0;

    while (1){
        if ((res = f_lseek(FilDL, ofsSubDirStart + (uint32_t)idxSubDir * RECLEN_DIRLIST04)) == FR_OK &&
            (res = f_read_DIRLIST04FILE(FilDL, RecDirList)) == FR_OK){
            
            DisplayDirectoryName(RecDirList);
            dirno = rowSubDirStart + idxSubDir - 1;
            numSndFile = *(RecDirList + DL_NUM_SNDFILE);
            recNoFirstSubdir = *((uint16_t *)(RecDirList + DL_RECNO_SUBDIR_START));
            lcd_icon_for_subdirectory(numSndFile, recNoFirstSubdir);
            
            ret = PauseAndSelectItem();
            if (ret == SWRIGHT_PRESSED){     // RIGHT
                idxSubDir++;
                if (idxSubDir == numSubDir) idxSubDir = 0;
            } else if (ret == SWLEFT_PRESSED){      // LEFT
                if (idxSubDir == 0) idxSubDir = numSubDir - 1;
                else idxSubDir--;
            } else if (ret == SWUP_PRESSED){        // UP
                uint16_t    r;
                r = *((uint16_t *)(RecDirList + DL_RECNO_GRANDPARENTDIR));   // rec no of grand parent
                // Go to grand parent record
                res = f_lseek(FilDL, r * RECLEN_DIRLIST04);
                if (res != FR_OK) return res;
                // Read grand parent record
                res = f_read_DIRLIST04FILE(FilDL, RecDirList);
                if (res != FR_OK) return res;
                // the first "uncle" record
                r = *((uint16_t *)(RecDirList + DL_RECNO_SUBDIR_START));
                if (r > 1){
                    rowSubDirStart = r;
                    ofsSubDirStart = (uint32_t)rowSubDirStart * RECLEN_DIRLIST04;
                    numSubDir = *(RecDirList + DL_NUM_SUBDIR);
                    // pop idxSubDir from stack -> This is parent
                    idxSubDir = stackIdxSubDir[--idxDirDepth];
                    if (idxSubDir >= numSubDir) idxSubDir = 0;
                }
            } else if (ret == SWDOWN_PRESSED){      // DOWN
                if (recNoFirstSubdir > 1){
                    rowSubDirStart = recNoFirstSubdir;
                    ofsSubDirStart = (uint32_t)rowSubDirStart * RECLEN_DIRLIST04;
                    numSubDir = *(RecDirList + DL_NUM_SUBDIR);
                    // push current idxSubDir to stack
                    stackIdxSubDir[idxDirDepth++] = idxSubDir;
                    idxSubDir = 0;
                } else if (numSndFile != 0){      // Have a MP3 file ?
                    ret = PlayOrSelectMp3InSubdirectory(RecDirList, dirno, numSndFile, 1, PS_SELECT);
                    if (ret == POWEROFF){
                        break;
                    }
                }
            } else if (ret == SWPLAY_PRESSED){             // PLAY
                if (numSndFile != 0){      // Have a MP3 file ?
                    ret = PlayOrSelectMp3InSubdirectory(RecDirList, dirno, numSndFile, 1, PS_PLAY);
                    if (ret == POWEROFF){
                        break;
                    }
                }
            } else if (ret == SWPOFF_PRESSED){      // POFF
                ret = POWEROFF;
                break;
            }
        } else {
            return res;
        }
    }
    
    f_close(FilDL);
    return ret;
}


int8
PauseAndSelectItem(){
    int8     ret;

    AutoPoffTmr = 0;
    AutoPoff = 1;

loop_PauseAndSelectItem:
    if (EVENT_SWPLAY && CHATTERING_TMR == 0){
        EVENT_SWPLAY = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWPLAY_PRESSED;
    }
    if (EVENT_SWRIGHT && CHATTERING_TMR == 0){
        EVENT_SWRIGHT = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWRIGHT_PRESSED;
    }
    if (EVENT_SWLEFT && CHATTERING_TMR == 0){
        EVENT_SWLEFT = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWLEFT_PRESSED;
    }
    if (EVENT_SWUP && CHATTERING_TMR == 0){
        EVENT_SWUP = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWUP_PRESSED;
    }
    if (EVENT_SWDOWN && CHATTERING_TMR == 0){
        EVENT_SWDOWN = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWDOWN_PRESSED;
    }
    if (EVENT_SWTEMPO && CHATTERING_TMR == 0){
        EVENT_SWTEMPO = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWTEMPO_PRESSED;
    }
    if (EVENT_SWTREBLE && CHATTERING_TMR == 0){
        EVENT_SWTREBLE = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWTREBLE_PRESSED;
    }
    if (EVENT_SWPOFF && CHATTERING_TMR == 0){
        EVENT_SWPOFF = 0;
        EVENT_SWANY = 0;
        AutoPoff = 0;
        return SWPOFF_PRESSED;
    }

    if (AutoPoffTmr == 1000){
        return SWPOFF_PRESSED;
    }

    goto loop_PauseAndSelectItem;

    return ret;
}

int8
PlayOrSelectMp3InSubdirectory(char *RecDirList, uint16_t dirno, uint16_t numsndfile, uint8_t start_trackno, playorselect_t playorselect){
    FRESULT     res;
    int8        ret;
    FIL         fil;
    char        RecMp3List[RECLEN_MP3LIST04];   // for MP3 list file and VOLUME file
    uint8_t     trackno;
    UINT        br;
    uint8_t     volinit;
    uint8_t     dirlen;
    BOOLEAN     execplayflg = 0;
#ifdef SUPPORT_RANDOM_TRACK
    uint8_t     random_trackno;

    if (PlayMode == PM_RANDOM_TRACK && playorselect == PS_PLAY){
        random_trackno = rand() % numsndfile + 1;
    }
#endif

    dirlen = *(RecDirList + DL_LENFULLDIR);
    
    // Volume file
    volinit = DEFAULT_VOLUME;
    *(RecDirList + DL_FULLDIR + dirlen) = '/';
    strcpy(RecDirList + DL_FULLDIR + dirlen + 1, VOLUME04FILE);
    res = f_open(&fil, RecDirList + DL_FULLDIR, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        if (f_read(&fil, RecMp3List, RECLEN_VOLUME04, &br) == FR_OK && br == RECLEN_VOLUME04){
            //if (RecMp3List[VF_VOLUME] != 0xFF){
                volinit = RecMp3List[VF_VOLUME];
                if (volinit <= MAX_VOLUME && volinit >= MIN_VOLUME){
                    VOLUME = volinit;
                }
            //}
        }
        f_close(&fil);
    }

    VS1033RESET();
    InitPitchShifter();

    // MP3 file list
    *(RecDirList + DL_FULLDIR + dirlen) = '/';
    strcpy(RecDirList + DL_FULLDIR + dirlen + 1, SNDFILELIST04FILE);
    res = f_open(&fil, RecDirList + DL_FULLDIR, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        ErrorLog(LogId, ERRORCODE_OPEN_MP3LISTFILE, res, dirno, 0);
        DispNoMp3List();
        return res;
    }

    trackno = 0;
    while (f_read(&fil, RecMp3List, RECLEN_MP3LIST04, &br) == FR_OK && br == RECLEN_MP3LIST04){
        // Track-No
        trackno++;
        if (PlayMode == PM_RANDOM_DIR){
            if (trackno < start_trackno) continue;
            if (trackno >= start_trackno) start_trackno = 1;
#ifdef SUPPORT_RANDOM_TRACK
        } else if (PlayMode == PM_RANDOM_TRACK){
            if (trackno != random_trackno) continue;
#endif
        }

        // select action
        if (PlayMode == PM_SELECT && playorselect == PS_SELECT){
            DisplayTitleName(RecMp3List);
            ret = PauseAndSelectItem();
            if (ret == SWPLAY_PRESSED){
                playorselect = PS_PLAY;
            } else if (ret == SWLEFT_PRESSED){
                if (trackno == 1){
                    trackno = numsndfile - 1;
                } else {
                    trackno -= 2;
                }
                f_lseek(&fil, trackno*RECLEN_MP3LIST04);
                continue;
            } else if (ret == SWRIGHT_PRESSED){
                if (trackno == numsndfile){
                    trackno = 0;
                    f_lseek(&fil, trackno*RECLEN_MP3LIST04);
                }
                continue;
            } else if (ret == SWUP_PRESSED){
                ret = PLAY_OTHER_DIR;
                break;
            } else if (ret == SWPOFF_PRESSED){
                ret = POWEROFF;
                break;
            } else {
                continue;
            }
        }

        // Mp3 file full path
        *(RecDirList + DL_FULLDIR + dirlen) = '/';
        strcpy(RecDirList + DL_FULLDIR + dirlen + 1, RecMp3List + ML_FILENAME);
        // Play a MP3 file
        ret = PlayOneMp3(RecDirList + DL_FULLDIR, dirno, trackno, RecDirList, RecMp3List);
        if (ret == PLAY_OTHER_DIR){
            break;
        //} else if (ret == PLAY_NEXT_MP3){
        } else if (ret == POWEROFF){
            break;
        }

        execplayflg = 1; 
    }
    f_close(&fil);

    // Save volume file
    if (execplayflg == 1 && VOLUME != volinit){     // at least one MP3 was played
        *(RecDirList + DL_FULLDIR + dirlen) = '/';
        strcpy(RecDirList + DL_FULLDIR + dirlen + 1, VOLUME04FILE);
        res = f_open(&fil, RecDirList + DL_FULLDIR, FA_READ | FA_WRITE | FA_OPEN_EXISTING);
        if (res == FR_OK){
            if (f_read(&fil, RecMp3List, RECLEN_VOLUME04, &br) == FR_OK){
                RecMp3List[VF_VOLUME] = VOLUME;
                RecMp3List[VF_COUNT]++;
                f_lseek(&fil, 0);
                f_write(&fil, RecMp3List, RECLEN_VOLUME04, &br);
            }
            f_close(&fil);
        }
    }

    return (ret);
}

/*
 *  mp3file : MP3 file fullpath
 *  dirno   : Dir No
 *
 */
int8
PlayOneMp3(const char *mp3file, uint16_t dirno, uint8_t trackno, const char *DirListRec, const char *RecMp3List){
    FRESULT     res;
    FIL         fil;
    UINT        br;
    int         sendbyte;
    int         continue_flag, ret;
    _Fwdfun     sendfunc;
    BOOLEAN     changedisp = FALSE;
    BOOLEAN     dispchanged = FALSE;
    uint16_t    dispchangecnt = 0;

    ret = FR_OK;
    res = f_open(&fil, mp3file, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        ErrorLog(LogId, ERRORCODE_OPEN_MP3FILE, res, dirno, trackno);
        DispNoMp3();
        return res;
    }
    DisplayDirectoryName(DirListRec);

    //VS1033RESET();
    //InitPitchShifter();
    sendbyte = SEND_SIZE;
    continue_flag = 1;
    sendfunc = VS1033SENDDATA;

    if (PonRestart == 1){       // POWER ON RESTART
        PonRestart = 0;
        VS1033SENDZEROS(2048);
        if (f_lseek(&fil, *(uint32_t *)(PlayDirHist + HS_LASTBYTE)) != FR_OK){      // go to restart point
            f_lseek(&fil, 0);
        }
    }
    VS1033VOLUME(VOLUME);

    *(uint16_t *)(PlayDirHist + HS_LASTDIR) = dirno;
    *(uint16_t *)(PlayDirHist + HS_LASTTRACK) = trackno;
    *(uint32_t *)(PlayDirHist + HS_LASTBYTE) = f_tell(&fil);

    while (continue_flag) {
        /*
        res = f_read(&fil, MP3DATA, MP3_BUFFER_SIZE, &br);
        if (res != FR_OK || br == 0) break;
        VS1033SENDDATA(MP3DATA, br);
        */

        // wait for data request from VS1033
        while(!(input(MP3_DREQ))){
            if (changedisp == TRUE){
                dispchanged = TRUE;
                changedisp = FALSE;
                DisplayTitleName(RecMp3List);
            }
        }

        // Read data from SD and send to VS1033
        res = f_forward(&fil, sendfunc, SEND_SIZE, &br);
        if (res != FR_OK || br == 0) break;

        // LCD display change control
        if (dispchanged == FALSE){
            dispchangecnt++;
            if (dispchangecnt == 0x2000){   // after (SEND_SIZE*0x2000)=262KByte data sent
                changedisp = TRUE;
            }
        }

        //if (br < MP3_BUFFER_SIZE){
        if (br < SEND_SIZE){
            continue_flag = 0;
        } else if (EVENT_SWANY == 1 && CHATTERING_TMR == 0){
            EVENT_SWANY = 0;
            if (EVENT_SWUP /*&& CHATTERING_TMR == 0*/){    // UP -> Volume up
                EVENT_SWUP = 0;
                if (VOLUME <=(STEP_VOLUME + MIN_VOLUME)){
                    VOLUME = MIN_VOLUME;
                } else {
                    VOLUME -= STEP_VOLUME;
                }
                //VS1033COMM(11,((uint16_t)VOLUME<<8)|((uint16_t)VOLUME));
                VS1033VOLUME(VOLUME);
                WriteEepromUint16(EEPROM_ADDR_VOL, VOLUME);
            } else if (EVENT_SWDOWN){  // DOWN -> Volume down
                EVENT_SWDOWN = 0;
                VOLUME += STEP_VOLUME;
                if (VOLUME > MAX_VOLUME){
                    VOLUME = MAX_VOLUME;
                }
                //VS1033COMM(11,((uint16_t)VOLUME<<8)|((uint16_t)VOLUME));
                VS1033VOLUME(VOLUME);
                WriteEepromUint16(EEPROM_ADDR_VOL, VOLUME);
            } else if (EVENT_SWPLAY){   // PLAY -> other dir
                EVENT_SWPLAY = 0;
                if (SW_CONTINUE_COUNT == 0){
                    ret = PLAY_NEXT_MP3;
                } else {    // long time push
                    ret = PLAY_OTHER_DIR;
                }
                SW_CONTINUE_COUNT = 0;
                continue_flag = 0;
            } else if(EVENT_SWRIGHT){    // RIGHT -> FF
                EVENT_SWRIGHT = 0;
                VS1033SENDZEROS(2048);
                f_lseek(&fil, f_tell(&fil) + FF_REW_SIZE);      // if overrun ?
            } else if(EVENT_SWLEFT){     // LEFT -> rewind
                EVENT_SWLEFT = 0;
                VS1033SENDZEROS(2048);
                if (f_tell(&fil) > FF_REW_SIZE){
                    f_lseek(&fil, f_tell(&fil) - FF_REW_SIZE);
                } else {
                    f_lseek(&fil, 0);
                }
            } else if (EVENT_SWTEMPO){      // TEMPO -> change tempo
                int16   tempo;
                EVENT_SWTEMPO = 0;
                PitchShifterMode++;
                if (PitchShifterMode == 1) tempo = -10923;  // 1.50
                //else if (PitchShifterMode == 2) tempo = -9362;  // 1.75
                else if (PitchShifterMode == 2) tempo = -8192;  // 2.0
                else {
                    PitchShifterMode = 0;
                    tempo = -16384;   // normal speed
                }
                VS1033COMM(0x0C, tempo);       // SCI_AICTRL0 = -16384/ratio
            } else if (EVENT_SWTREBLE){
                EVENT_SWTREBLE = 0;
                BTControl++;
                if (BTControl == 1){
                    ST_AMP = 0x08;
                    ST_FREQ = 0x03;     // 3kHz
                } else if (BTControl == 2){
                    ST_AMP = 0x08;
                    ST_FREQ = 0x02;     // 2kHz
                } else if (BTControl == 3){
                    ST_AMP = 0x08;
                    ST_FREQ = 0x01;     // 1kHz
                } else {
                    BTControl = 0;
                    ST_AMP = 0x00;      // Normal
                    ST_FREQ = 0x00;     // Normal
                }
                VS1033COMM(2, ((ST_AMP << 12) | (ST_FREQ << 8) | (SB_AMP << 4) | SB_FREQ));
                lcd_icon_bt_control(BTControl);
            } else if (EVENT_SWPOFF){
                EVENT_SWPOFF = 0;
                *(uint32_t *)(PlayDirHist + HS_LASTBYTE) = f_tell(&fil);
                ret = POWEROFF;
                continue_flag = 0;
            }
        }
    }

    f_close(&fil);
    return ret;
}

/*
 *  Display Directory Name / Album Name
 */
void DisplayDirectoryName(char *RecDirList){
    i2c_start_lcd();
    lcd_locate(0, 0);
    lcd_println(&RecDirList[DL_DISPLAYNAME]);   // Album name
    lcd_locate(1, 0);
    lcd_println(&RecDirList[DL_DISPLAYNAME + 16]);   // Album name
    i2c_stop();
}

/*
 *  Display MP3 tile
 */
void DisplayTitleName(char *RecMp3List){
    i2c_start_lcd();
    lcd_locate(0, 0);
    lcd_println(RecMp3List + ML_TRACKNO);
    lcd_println(RecMp3List + ML_TITLENAME);
    lcd_locate(1, 0);
    lcd_println(RecMp3List + ML_TITLENAME + 16 - 2);
    i2c_stop();
}

void InitApp(void)
{
    uint32_t    seed;
    
    setup_adc(ADC_OFF);
    setup_ccp1(CCP_OFF);

    // 18F2580 (INTIO2 / port function on RA6 and RA7)
    setup_oscillator(OSC_32MHZ);

    output_high(PIN_POWER_ON);
    output_float(PIN_POWER_OFF);
    output_high(PIN_LED);
    
    output_float(MP3_DREQ);     //

    output_float(PIN_SWPLAY);       //
    output_float(PIN_SWRIGHT);       //
    output_float(PIN_SWLEFT);       //
    output_float(PIN_SWUP);       //
    output_float(PIN_SWDOWN);       //
    output_float(PIN_SWTEMPO);       //
    output_float(PIN_SWTREBLE);       //

    output_float(PIN_C4);       // SPI/SDI

    port_b_pullups(TRUE);

    delay_ms(100);

    lcd_init();     // Lcd_I2C.c
    
    setup_timer_1(T1_INTERNAL | T1_DIV_BY_8);
    set_timer1(TMR1REG_START);
    clear_interrupt(INT_TIMER1);
    enable_interrupts(INT_TIMER1);
    enable_interrupts(GLOBAL);

    // set seed of srand
    seed = ReadEepromUint16(EEPROM_ADDR_SEED);
    if (seed == 0x0000FFFF) seed = 1;
    else seed++;
    srand(seed);
    WriteEepromUint16(EEPROM_ADDR_SEED, seed);
    LogId = (seed & 0xFF);
}

uint16_t ReadEepromUint16(uint8_t address){
    uint16_t    h;
    h = read_eeprom(address + 1);       // PCH built-in function
    h <<= 8;
    h |= read_eeprom(address);

    return h;
}

void WriteEepromUint16(uint8_t address, uint16_t data){
    write_eeprom(address, data & 0xFF);
    write_eeprom(address + 1, (data >> 8) & 0xFF);
}

#if 0
uint32_t ReadEepromUint32(uint8_t address){
    uint32_t    h;
    h = read_eeprom(address + 3);       // PCH built-in function
    h <<= 8;
    h |= read_eeprom(address + 2);       // PCH built-in function
    h <<= 8;
    h |= read_eeprom(address + 1);       // PCH built-in function
    h <<= 8;
    h |= read_eeprom(address);

    return h;
}

void WriteEepromUint32(uint8_t address, uint32_t data){
    write_eeprom(address, data & 0xFF);
    write_eeprom(address + 1, (data >> 8) & 0xFF);
    write_eeprom(address + 2, (data >> 16) & 0xFF);
    write_eeprom(address + 3, (data >> 24) & 0xFF);
}
#endif

void InitPitchShifter(void) {
    int16       i = 0;
    FIL         fin;
    FRESULT     res;
    uint16_t    num;
    UINT        br;

    // get number of data
    res = f_open(&fin, PITCHSHIFTERFILE, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK) return;
    res = f_read(&fin, &num, 2, &br);
    if (res != FR_OK || br != 2) return;

    while (i<num) {
        //  while (i<sizeof(plugin)/sizeof(plugin[0])) {
        uint16_t addr, n, val;
        //    unsigned short addr, n, val;
        //addr = plugin[i++];
        //n = plugin[i++];
        res = f_read(&fin, &addr, 2, &br);
        i++;
        res = f_read(&fin, &n, 2, &br);
        i++;
        if (n & 0x8000U) { /* RLE run, replicate n samples */
            n &= 0x7FFF;
            //val = plugin[i++];
            res = f_read(&fin, &val, 2, &br);
            i++;
            while (n--) {
                VS1033COMM(addr, val);
                //        WriteVS10xxRegister(addr, val);
            }
        } else {           /* Copy run, copy n samples */
            while (n--) {
                //val = plugin[i++];
                res = f_read(&fin, &val, 2, &br);
                i++;
                VS1033COMM(addr, val);
                //        WriteVS10xxRegister(addr, val);
            }
        }
    }

    f_close(&fin);

    VS1033COMM(0x0C, -16384);       // SCI_AICTRL0 : Normal speed
    VS1033COMM(0x0A, 0x30);         // SCI_AIADDR
}

void ReadHistory(FIL *Fil){
    FRESULT     res;
    UINT        br;

    res = f_open(Fil, HIST04FILE, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        f_read(Fil, PlayDirHist, RECLEN_HISTFILE04, &br);
        f_close(Fil);
    }
}

void SaveHistory(FIL *Fil){
    FRESULT     res;
    UINT        br;

    res = f_open(Fil, HIST04FILE, FA_WRITE | FA_OPEN_EXISTING);
    if (res == FR_OK){
        res = f_write(Fil, PlayDirHist, RECLEN_HISTFILE04, &br);
        f_close(Fil);
    }

    if (res != FR_OK){
        ErrorLog(LogId, ERRORCODE_SAVE_HISTORYFILE, res, 0, 0);
    }
}

void ErrorLog(uint8_t id, uint8_t errorcode, uint8_t errorvalue, uint16_t dirno, uint8_t trackno){
    FIL         FilLog;
    uint8_t     RecErrlog[RECLEN_ERRLOG04];
    int8        i, res;
    UINT        br;

    res = f_open(&FilLog, ERRLOG04FILE, FA_WRITE | FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
        for (i=0; i<MAX_ERRLOG_COUNT; i++){
            f_read(&FilLog, RecErrlog, RECLEN_ERRLOG04, &br);
            if (*(RecErrlog + EL_ERRORCODE) == 0){
                *(RecErrlog + EL_ID) = id;
                *(RecErrlog + EL_ERRORCODE) = errorcode;
                *(RecErrlog + EL_ERRORVALUE) = errorvalue;
                *((uint16_t *)(RecErrlog + EL_DIRNO)) = dirno;
                *(RecErrlog + EL_TRACKNO) = trackno;

                f_lseek(&FilLog, i * RECLEN_ERRLOG04);
                f_write(&FilLog, RecErrlog, RECLEN_ERRLOG04, &br);
                break;
            }
        }
        f_close(&FilLog);
    }
}

void DispNoDirList(){
    i2c_start_lcd();
    lcd_locate(1, 0); lcd_println("NO DIR LIST     ");
    i2c_stop();
    delay_ms(ERROR_WAIT_TIME);
}

void DispNoMp3List(){
    i2c_start_lcd();
    lcd_locate(1, 0); lcd_println("NO MP3 LIST     ");
    i2c_stop();
    delay_ms(ERROR_WAIT_TIME);
}

void DispNoMp3(){
    i2c_start_lcd();
    lcd_locate(1, 0); lcd_println("NO MP3       ");
    i2c_stop();
    delay_ms(ERROR_WAIT_TIME);
}

#if 0
void DispNumForDebug(uint32_t v){
    char    buf[9];
    int     i;

    for (i=7; i>=0; i--){
        buf[7 - i] = itoh(0x0f & (v >> (i*4)));     // Shift operator don't work ?
    }
    buf[8] = '\0';

    i2c_start_lcd();
    lcd_locate(1, 0); lcd_println(buf);
    i2c_stop();
    delay_ms(ERROR_WAIT_TIME);
}
#endif
